"use strict";
/*
  Copyright © 2017 Kerry Shetline, kerry@shetline.com

  MIT license: https://opensource.org/licenses/MIT

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit
  persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
  Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var ks_math_1 = require("./ks-math");
var util_1 = require("util");
var Unit;
(function (Unit) {
    Unit[Unit["RADIANS"] = 0] = "RADIANS";
    Unit[Unit["DEGREES"] = 1] = "DEGREES";
    Unit[Unit["ARC_MINUTES"] = 2] = "ARC_MINUTES";
    Unit[Unit["ARC_SECONDS"] = 3] = "ARC_SECONDS";
    Unit[Unit["HOURS"] = 4] = "HOURS";
    Unit[Unit["HOUR_ANGLE_MINUTES"] = 5] = "HOUR_ANGLE_MINUTES";
    Unit[Unit["HOUR_ANGLE_SECONDS"] = 6] = "HOUR_ANGLE_SECONDS";
    Unit[Unit["ROTATIONS"] = 7] = "ROTATIONS";
    Unit[Unit["GRADS"] = 8] = "GRADS";
})(Unit = exports.Unit || (exports.Unit = {}));
var Mode;
(function (Mode) {
    Mode[Mode["RANGE_LIMIT_SIGNED"] = 0] = "RANGE_LIMIT_SIGNED";
    Mode[Mode["RANGE_LIMIT_NONNEGATIVE"] = 1] = "RANGE_LIMIT_NONNEGATIVE";
    Mode[Mode["RANGE_UNLIMITED"] = 2] = "RANGE_UNLIMITED";
})(Mode = exports.Mode || (exports.Mode = {}));
exports.PI = Math.PI;
exports.HALF_PI = exports.PI / 2.0;
exports.TWO_PI = exports.PI * 2.0;
exports.FMT_DD = 0x01;
exports.FMT_HH = 0x01;
exports.FMT_DDD = 0x02;
exports.FMT_MINS = 0x04;
exports.FMT_SECS = 0x08;
exports.FMT_SIGNED = 0x10;
function convertToRadians(angle, unit) {
    switch (unit) {
        case Unit.RADIANS:
            return angle;
        case Unit.DEGREES:
            return angle / 180.0 * exports.PI;
        case Unit.ARC_MINUTES:
            return angle / 10800.0 * exports.PI;
        case Unit.ARC_SECONDS:
            return angle / 648000.0 * exports.PI;
        case Unit.HOURS:
            return angle / 12.0 * exports.PI;
        case Unit.HOUR_ANGLE_MINUTES:
            return angle / 720.0 * exports.PI;
        case Unit.HOUR_ANGLE_SECONDS:
            return angle / 43200.0 * exports.PI;
        case Unit.ROTATIONS:
            return angle * exports.TWO_PI;
        case Unit.GRADS:
            return angle / 200.0 * exports.PI;
    }
    return NaN;
}
exports.convertToRadians = convertToRadians;
function convertFromRadians(angle, unit) {
    switch (unit) {
        case Unit.RADIANS:
            return angle;
        case Unit.DEGREES:
            return angle * 180.0 / exports.PI;
        case Unit.ARC_MINUTES:
            return angle * 10800.0 / exports.PI;
        case Unit.ARC_SECONDS:
            return angle * 648000.0 / exports.PI;
        case Unit.HOURS:
            return angle * 12.0 / exports.PI;
        case Unit.HOUR_ANGLE_MINUTES:
            return angle * 720.0 / exports.PI;
        case Unit.HOUR_ANGLE_SECONDS:
            return angle * 43200.0 / exports.PI;
        case Unit.ROTATIONS:
            return angle / exports.TWO_PI;
        case Unit.GRADS:
            return angle * 200.0 / exports.PI;
    }
    return NaN;
}
exports.convertFromRadians = convertFromRadians;
var Angle = /** @class */ (function () {
    function Angle(angle, unit, mode) {
        if (angle === void 0) { angle = 0; }
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        this.cached_sin = 2.0;
        this.cached_cos = 2.0;
        this.cached_tan = 0.0;
        if (unit === undefined)
            unit = Unit.RADIANS;
        if (mode === Mode.RANGE_LIMIT_SIGNED)
            this.angle = ks_math_1.mod2(convertToRadians(angle, unit), exports.TWO_PI);
        else if (mode === Mode.RANGE_LIMIT_NONNEGATIVE)
            this.angle = ks_math_1.mod(convertToRadians(angle, unit), exports.TWO_PI);
        else
            this.angle = convertToRadians(angle, unit);
    }
    Angle.asin = function (x) {
        return new Angle(Math.asin(x));
    };
    Angle.asin_nonneg = function (x) {
        return new Angle(Math.asin(x), Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.acos = function (x) {
        return new Angle(Math.acos(x));
    };
    Angle.atan = function (x) {
        return new Angle(Math.atan(x));
    };
    Angle.atan_nonneg = function (x) {
        return new Angle(Math.atan(x), Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.atan2 = function (y, x) {
        return new Angle(Math.atan2(y, x));
    };
    Angle.atan2_nonneg = function (y, x) {
        return new Angle(Math.atan2(y, x), Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Object.defineProperty(Angle.prototype, "radians", {
        get: function () {
            return this.angle;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "degrees", {
        get: function () {
            return convertFromRadians(this.angle, Unit.DEGREES);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "arcMinutes", {
        get: function () {
            return convertFromRadians(this.angle, Unit.ARC_MINUTES);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "arcSeconds", {
        get: function () {
            return convertFromRadians(this.angle, Unit.ARC_SECONDS);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "hours", {
        get: function () {
            return convertFromRadians(this.angle, Unit.HOURS);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "rotations", {
        get: function () {
            return convertFromRadians(this.angle, Unit.ROTATIONS);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "grads", {
        get: function () {
            return convertFromRadians(this.angle, Unit.GRADS);
        },
        enumerable: true,
        configurable: true
    });
    Angle.prototype.getAngle = function (unit) {
        if (unit === void 0) { unit = Unit.RADIANS; }
        return convertFromRadians(this.angle, unit);
    };
    Object.defineProperty(Angle.prototype, "sin", {
        get: function () {
            if (this.cached_sin > 1.0)
                this.cached_sin = Math.sin(this.angle);
            return this.cached_sin;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "cos", {
        get: function () {
            if (this.cached_cos > 1.0)
                this.cached_cos = Math.cos(this.angle);
            return this.cached_cos;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Angle.prototype, "tan", {
        get: function () {
            if (this.angle === 0.0)
                return 0.0;
            else if (this.cached_tan === 0.0)
                this.cached_tan = Math.tan(this.angle);
            return this.cached_tan;
        },
        enumerable: true,
        configurable: true
    });
    Angle.prototype.add = function (angle2, mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(this.angle + angle2.angle, Unit.RADIANS, mode);
    };
    Angle.prototype.add_nonneg = function (angle2) {
        return new Angle(this.angle + angle2.angle, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.subtract = function (angle2, mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(this.angle - angle2.angle, Unit.RADIANS, mode);
    };
    Angle.prototype.subtract_nonneg = function (angle2) {
        return new Angle(this.angle - angle2.angle, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.complement = function (mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(exports.HALF_PI - this.angle, Unit.RADIANS, mode);
    };
    Angle.prototype.complement_nonneg = function () {
        return new Angle(exports.HALF_PI - this.angle, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.supplement = function (mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(exports.PI - this.angle, Unit.RADIANS, mode);
    };
    Angle.prototype.supplement_nonneg = function () {
        return new Angle(exports.PI - this.angle, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.opposite = function (mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(this.angle + exports.PI, Unit.RADIANS, mode);
    };
    Angle.prototype.opposite_nonneg = function () {
        return new Angle(this.angle + exports.PI, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.negate = function (mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(-this.angle, Unit.RADIANS, mode === undefined ? Mode.RANGE_UNLIMITED : mode);
    };
    // Sounds contradictory, doesn't it? Return whatever angle is 180 degrees away, as a non-negative value.
    Angle.prototype.negate_nonneg = function () {
        return new Angle(-this.angle, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.multiply = function (x, mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(this.angle * x, Unit.RADIANS, mode);
    };
    Angle.prototype.multiply_nonneg = function (x) {
        return new Angle(this.angle * x, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.divide = function (x, mode) {
        if (mode === void 0) { mode = Mode.RANGE_LIMIT_SIGNED; }
        return new Angle(this.angle / x, Unit.RADIANS, mode);
    };
    Angle.prototype.divide_nonneg = function (x) {
        return new Angle(this.angle / x, Unit.RADIANS, Mode.RANGE_LIMIT_NONNEGATIVE);
    };
    Angle.prototype.toString = function (format, precision) {
        return Angle.toStringAux(this.degrees, '\u00B0', '\'', '"', format, precision);
    };
    Angle.prototype.toSuffixedString = function (positiveSuffix, negativeSuffix, format, precision) {
        return Angle.toStringAux(ks_math_1.abs(this.degrees), '\u00B0', '\'', '"', format, precision) +
            (this.degrees < 0 ? negativeSuffix : positiveSuffix);
    };
    Angle.prototype.toHourString = function (format, precision) {
        return Angle.toStringAux(this.hours, 'h', 'm', 's', format, precision);
    };
    Angle.prototype.toTimeString = function (format, precision) {
        return Angle.toStringAux(this.hours, ':', format === exports.FMT_MINS ? '' : ':', '', format, precision, 2);
    };
    Angle.toStringAux = function (units, delim1, delim2, delim3, format, precision, unitsPadding) {
        if (unitsPadding === void 0) { unitsPadding = 0; }
        format = (format ? format : 0);
        var sxg = ((format & (exports.FMT_MINS | exports.FMT_SECS)) !== 0);
        if ((format & exports.FMT_DD) !== 0)
            unitsPadding = 2;
        else if ((format & exports.FMT_DDD) !== 0)
            unitsPadding = 3;
        if (util_1.isUndefined(precision)) {
            if (!util_1.isUndefined(format) && sxg)
                precision = 0;
            else
                precision = 3;
        }
        var sign = Math.sign(units);
        units = ks_math_1.abs(units);
        var result;
        if (sxg) {
            var pwr = ks_math_1.pow(10, precision);
            if ((format & exports.FMT_MINS) !== 0) {
                var mins = ks_math_1.round(units * 60 * pwr) / pwr;
                units = ks_math_1.floor(mins / 60);
                mins = mins % 60;
                result = units + delim1 + (mins < 10 ? '0' : '') + mins.toFixed(precision) + delim2;
            }
            else {
                var secs = ks_math_1.round(units * 3600 * pwr) / pwr;
                var mins = ks_math_1.floor(secs / 60);
                secs = secs % 60;
                units = ks_math_1.floor(mins / 60);
                mins = mins % 60;
                result = units + delim1 + (mins < 10 ? '0' : '') + mins + delim2
                    + (secs < 10 ? '0' : '') + secs.toFixed(precision) + delim3;
            }
        }
        else {
            result = units.toFixed(precision) + delim1;
        }
        if (unitsPadding) {
            var match = /^(\d+)\D/.exec(result);
            var padding = unitsPadding - match[1].length;
            for (var i = 0; i < padding; ++i)
                result = '0' + result;
        }
        if (sign < 0)
            result = '-' + result;
        else if ((format & exports.FMT_SIGNED) !== 0)
            result = '+' + result;
        return result;
    };
    Angle.ZERO = new Angle(0.0);
    Angle.RIGHT = new Angle(exports.HALF_PI);
    Angle.STRAIGHT = new Angle(exports.PI);
    return Angle;
}());
exports.Angle = Angle;
//# sourceMappingURL=angle.js.map