"use strict";
/*
  Copyright © 2017 Kerry Shetline, kerry@shetline.com

  MIT license: https://opensource.org/licenses/MIT

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit
  persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
  Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  This code is an adaption of code from: "Numerical Recipes, The Art of Scientific Computing"
  by W.H. Press, B.P. Flannery, S.A. Teukolsky and W.T. Vetterling. Cambridge University Press, 1986.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var ks_math_1 = require("./ks-math");
var CGOLD = 0.381966011250105; // (3 - sqrt(5)) / 2
var ZEPS = 1.0E-20;
var MinMaxFinder = /** @class */ (function () {
    function MinMaxFinder(minMaxSeekingFunction, tolerance, maxIterations, ax, bx, cx) {
        this.minMaxSeekingFunction = minMaxSeekingFunction;
        this.tolerance = tolerance;
        this.maxIterations = maxIterations;
        this.ax = ax;
        this.bx = bx;
        this.cx = cx;
        this._iterationCount = 0;
        this.isMin = true;
    }
    MinMaxFinder.prototype.getXAtMinMax = function () {
        var a;
        var b;
        var d = 0.0;
        var etemp;
        var fu;
        var fv;
        var fw;
        var p;
        var q;
        var r;
        var tol1;
        var tol2;
        var u;
        var v;
        var w;
        var x;
        var xm;
        var e = 0.0;
        var sign = 1.0;
        a = (this.ax < this.cx ? this.ax : this.cx);
        b = (this.ax > this.cx ? this.ax : this.cx);
        x = w = v = this.bx;
        this.fx = this.minMaxSeekingFunction(x);
        // Reverse the sign of the evaluated function if we're searching for a max rather than a min.
        if (this.fx > this.minMaxSeekingFunction(this.ax)) {
            this.isMin = false;
            this.fx *= -1;
            sign = -1.0;
        }
        else
            this.isMin = true;
        fw = fv = this.fx;
        this._iterationCount = 0;
        while (++this._iterationCount <= this.maxIterations) {
            xm = 0.5 * (a + b);
            tol1 = this.tolerance * ks_math_1.abs(x) + ZEPS;
            tol2 = 2.0 * tol1;
            if (ks_math_1.abs(x - xm) <= tol2 - 0.5 * (b - a)) {
                this.fx *= sign;
                return x;
            }
            if (ks_math_1.abs(e) > tol1) {
                r = (x - w) * (this.fx - fv);
                q = (x - v) * (this.fx - fw);
                p = (x - v) * q - (x - w) * r;
                q = 2.0 * (q - r);
                if (q > 0.0)
                    p = -p;
                q = ks_math_1.abs(q);
                etemp = e;
                e = d;
                if (ks_math_1.abs(p) >= ks_math_1.abs(0.5 * q * etemp) || p <= q * (a - x) || p >= q * (b - x)) {
                    e = (x >= xm ? a - x : b - x);
                    d = CGOLD * e;
                }
                else {
                    d = p / q;
                    u = x + d;
                    if (u - a < tol2 || b - u < tol2)
                        d = Math.sign(xm - x) * tol1;
                }
            }
            else {
                e = (x >= xm ? a - x : b - x);
                d = CGOLD * e;
            }
            u = (ks_math_1.abs(d) >= tol1 ? x + d : x + Math.sign(d) * tol1);
            fu = sign * this.minMaxSeekingFunction(u);
            if (fu <= this.fx) {
                if (u >= x)
                    a = x;
                else
                    b = x;
                v = w;
                w = x;
                x = u;
                fv = fw;
                fw = this.fx;
                this.fx = fu;
            }
            else {
                if (u < x)
                    a = u;
                else
                    b = u;
                if (fu <= fw || w === x) {
                    v = w;
                    w = u;
                    fv = fw;
                    fw = fu;
                }
                else if (fu <= fv || v === x || v === w) {
                    v = u;
                    fv = fu;
                }
            }
        }
        this.fx *= sign;
        return x;
    };
    Object.defineProperty(MinMaxFinder.prototype, "foundMaximum", {
        get: function () { return !this.isMin; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MinMaxFinder.prototype, "foundMinimum", {
        get: function () { return this.isMin; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MinMaxFinder.prototype, "lastY", {
        get: function () { return this.fx; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MinMaxFinder.prototype, "iterationCount", {
        get: function () { return this._iterationCount; },
        enumerable: true,
        configurable: true
    });
    return MinMaxFinder;
}());
exports.MinMaxFinder = MinMaxFinder;
//# sourceMappingURL=min-max-finder.js.map